// ==============================================================
//                 ORBITER MODULE: ITSAMFD
//                  Part of the ORBITER SDK
//           Copyright (C) 2002-2003 Martin Schweiger
//                   All rights reserved
//
// ITSAMFD.cpp
// Reference implementation of generic user-defined MFD mode
// ==============================================================

#define STRICT
#define ORBITER_MODULE
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "orbitersdk.h"
#include "ITSArrayMFD.h"

// ==============================================================
// Global variables



char vesname[256];
char vesname2[256][10];

OBJHANDLE MyVes;

const DWORD COMM_SEND_POWER = 2300;		// returns power output Watts
const DWORD COMM_SOLARRAY_IDENT = 2301; // returns deploy_status of vessel, identified as array if returns > 0
const DWORD COMM_AUTOSUN_STATUS = 2302; // returns auto sun tracking status on/off
const DWORD COMM_SARJ_STATUS = 2303; // returns sarj rotation status
const DWORD COMM_ABBGA_STATUS = 2304;	// returns AB BGA rotation status
const DWORD COMM_CDBGA_STATUS = 2305;	// returns CD BGA rotation status
const DWORD COMM_ATT_STATUS = 2306;	// returns parent attachment status

double ident;
double ves_power[10];
double tot_power;
int dep_status;
int auto_sun_status;
int att_status;
int sarj_status;
int ab_bga_status;
int cd_bga_status;
int itsa_page = 0;

static struct {  // "ITSAMFD" parameters
	int mode;      // identifier for new MFD mode
} g_ITSAMFD;



// ==============================================================
// API interface

DLLCLBK void InitModule (HINSTANCE hDLL)
{


	static char *name = "ITSArrayMFD";
	MFDMODESPEC spec;
	spec.name    = name;
	spec.key     = OAPI_KEY_V;
	spec.msgproc = ITSAMFD::MsgProc;


	g_ITSAMFD.mode = oapiRegisterMFDMode (spec);


}

DLLCLBK void ExitModule (HINSTANCE hDLL)
{
	oapiUnregisterMFDMode (g_ITSAMFD.mode);

}




DLLCLBK void opcPreStep (double SimT, double SimDT, double mjd)
{



}



// ==============================================================
// ITSAMFD implementation

ITSAMFD::ITSAMFD (DWORD w, DWORD h, VESSEL *vessel)
: GraphMFD (w, h, vessel)
{

	int vc = oapiGetVesselCount();
	for(int i=0;i<vc;i++)
	{
	OBJHANDLE ves = oapiGetVesselByIndex(i);
	VESSEL2* v =(VESSEL2*)oapiGetVesselInterface(ves);
	int vv = v->Version();
	if (vv == 0);
	else{
		dep_status = (v->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,false,(char *)&ident));
		if(dep_status > 0)
			{
			oapiGetObjectName(ves, vesname, 30);
			auto_sun_status = v->clbkConsumeBufferedKey(COMM_AUTOSUN_STATUS,false,(char *)&ident);
			att_status = v->clbkConsumeBufferedKey(COMM_ATT_STATUS,false,(char *)&ident);
			sarj_status = v->clbkConsumeBufferedKey(COMM_SARJ_STATUS,false,(char *)&ident);
			ab_bga_status = v->clbkConsumeBufferedKey(COMM_ABBGA_STATUS,false,(char *)&ident);
			cd_bga_status = v->clbkConsumeBufferedKey(COMM_CDBGA_STATUS,false,(char *)&ident);
			break;
			}
		}
	}

}

ITSAMFD::~ITSAMFD ()
{

}


// message parser
int ITSAMFD::MsgProc (UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam)
{
	switch (msg) {
	case OAPI_MSG_MFD_OPENED:
		return (int)(new ITSAMFD (LOWORD(wparam), HIWORD(wparam), (VESSEL*)lparam));
	}
	return 0;
}






bool ITSAMFD::ConsumeKeyBuffered (DWORD key)
{

	bool cbSelectVessel(void *id, char *str, void *data);

	switch (key)
	{

	case OAPI_KEY_B:
		oapiOpenInputBox("Enter Array Name", cbSelectVessel, 0, 30, (void *)this);
		return true;
	case OAPI_KEY_G:
		SendDeployStow();
		return true;
	case OAPI_KEY_K:
		SendAutoSunOnOff();
		return true;
	case OAPI_KEY_N:
		AttachGrapple();
		return true;
	case OAPI_KEY_M:
		AttachRelease();
		return true;

	case OAPI_KEY_1:
		RotSARJPos();
		return true;
	case OAPI_KEY_2:
		RotSARJStop();
		return true;
	case OAPI_KEY_3:
		RotSARJNeg();
		return true;

	case OAPI_KEY_4:
		RotABPos();
		return true;
	case OAPI_KEY_5:
		RotABStop();
		return true;
	case OAPI_KEY_6:
		RotABNeg();
		return true;

	case OAPI_KEY_7:
		RotCDPos();
		return true;
	case OAPI_KEY_8:
		RotCDStop();
		return true;
	case OAPI_KEY_9:
		RotCDNeg();
		return true;

	case OAPI_KEY_P:
		if (itsa_page == 0)itsa_page = 1;
		else if (itsa_page == 1)itsa_page = 0;
		InvalidateButtons();
		InvalidateDisplay();
		StoreStatus();
		return true;

}

	return false;
}

bool ITSAMFD::ConsumeButton (int bt, int event)
{

	if (!(event & PANEL_MOUSE_LBDOWN)) return false;
	if (itsa_page == 0)
	{
	static const DWORD btkey[6] = {OAPI_KEY_B, OAPI_KEY_G, OAPI_KEY_K, OAPI_KEY_N, OAPI_KEY_M, OAPI_KEY_P};

	if (bt < 6) return ConsumeKeyBuffered (btkey[bt]);
	else return false;
	}



	if (itsa_page == 1)
	{
	static const DWORD btkey[12] = {OAPI_KEY_1, OAPI_KEY_2, OAPI_KEY_3, NULL, NULL, OAPI_KEY_P, OAPI_KEY_4,
	OAPI_KEY_5, OAPI_KEY_6, OAPI_KEY_7, OAPI_KEY_8, OAPI_KEY_9};

	if (bt < 12) return ConsumeKeyBuffered (btkey[bt]);
	else return false;
	}

}

char *ITSAMFD::ButtonLabel (int bt)
{
	if (itsa_page == 0)
	{
	char *label[6] = {"SEL", "DEP", "SUN", "GRP", "RLS", "PG"};

	return (bt < 6 ? label[bt] : 0);
	}

	if (itsa_page == 1)
	{
	char *label[12] = {"SJ+", "SJo", "SJ-", NULL, NULL, "PG", "AB+", "ABo", "AB-", "CD+", "CDo", "CD-"};

	return (bt < 12 ? label[bt] : 0);
	}
}

int ITSAMFD::ButtonMenu (const MFDBUTTONMENU **menu) const
{
	if (itsa_page == 0)
	{
	static const MFDBUTTONMENU mnu[6] = {
		{"Select Array", 0, 'B'},
		{"Send Deploy/Pause/Stow Command", 0, 'G'},
		{"Toggle Auto Sun Tracking On/Off", 0, 'K'},
		{"Grapple Attach Point", 0, 'N'},
		{"Release Attach Point", 0, 'M'},
		{"Next Page", 0, 'P'}
	};
	if (menu) *menu = mnu;
	return 6;
	}

	if (itsa_page == 1)
	{
	static const MFDBUTTONMENU mnu[12] = {
		{"Rot SARJ +", 0, '1'},
		{"Rot SARJ Stop", 0, '2'},
		{"Rot SARJ -", 0, '3'},
		{NULL},
		{NULL},
		{"Prev. Page", 0, 'P'},
		{"Rot AB BGA +", 0, '4'},
		{"Rot AB BGA Stop", 0, '5'},
		{"Rot AB BGA -", 0, '6'},
		{"Rot CD BGA +", 0, '7'},
		{"Rot CD BGA Stop", 0, '8'},
		{"Rot CD BGA -", 0, '9'}

	};
	if (menu) *menu = mnu;
	return 12;
	}
}


bool cbSelectVessel(void *id, char *str, void *data)
{
	return (((ITSAMFD *)data)->SelectVessel(str));
}


bool ITSAMFD::SelectVessel(char *Name)
{
	MyVes = oapiGetVesselByName(Name);

	if (MyVes)
	{
	VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(MyVes);
	int vv = v->Version();
	if (vv == 0)return false;
	else
		{
		if((v->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,false,(char *)&ident)) > 0)
			{
			strcpy(vesname,Name);
			InvalidateDisplay();
			StoreStatus();
			return true;
			}
		else return false;
		}
	}

	return false;

}

void ITSAMFD::GetArray (void)
{

	int di = 0;
	int vc = oapiGetVesselCount();
	for(int i=0;i<vc;i++)
	{
	OBJHANDLE ves = oapiGetVesselByIndex(i);
	VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(ves);
	int vv = v->Version();
	if (vv == 0);
	else
		{
		if((v->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,false,(char *)&ident)) > 0)
			{
			oapiGetObjectName(ves, vesname2[di], 20);
			ves_power[di] = v->clbkConsumeBufferedKey(COMM_SEND_POWER,false,(char *)&ident);
			if (!strnicmp(vesname,vesname2[di],20))
			{
			dep_status = v->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,false,(char *)&ident);
			auto_sun_status = v->clbkConsumeBufferedKey(COMM_AUTOSUN_STATUS,false,(char *)&ident);
			att_status = v->clbkConsumeBufferedKey(COMM_ATT_STATUS,false,(char *)&ident);
			sarj_status = v->clbkConsumeBufferedKey(COMM_SARJ_STATUS,false,(char *)&ident);
			ab_bga_status = v->clbkConsumeBufferedKey(COMM_ABBGA_STATUS,false,(char *)&ident);
			cd_bga_status = v->clbkConsumeBufferedKey(COMM_CDBGA_STATUS,false,(char *)&ident);
			}
			di = di+1;
			}

		}
	}

}



void ITSAMFD::SendDeployStow(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
	{
	if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,false,(char *)&ident)) > 0)
		{
		ves->clbkConsumeBufferedKey(OAPI_KEY_G,true,(char *)&dbl);
		InvalidateDisplay();
		}
	}
}
}

void ITSAMFD::SendAutoSunOnOff(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
{
if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,true,(char *)&ident)) > 0)
	{
	ves->clbkConsumeBufferedKey(OAPI_KEY_K,true,(char *)&dbl);
	InvalidateDisplay();
	}
}
}
}

void ITSAMFD::AttachGrapple(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
{
if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,true,(char *)&ident)) > 0)
	{
	ves->clbkConsumeBufferedKey(OAPI_KEY_N,true,(char *)&dbl);
	InvalidateDisplay();
	}
}
}
}

void ITSAMFD::AttachRelease(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
{
if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,true,(char *)&ident)) > 0)
	{
	ves->clbkConsumeBufferedKey(OAPI_KEY_M,true,(char *)&dbl);
	InvalidateDisplay();
	}
}
}
}


void ITSAMFD::RotSARJPos(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
{
if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,true,(char *)&ident)) > 0)
	{
	ves->clbkConsumeBufferedKey(OAPI_KEY_NUMPAD1,true,(char *)&dbl);
	InvalidateDisplay();
	}
}
}
}

void ITSAMFD::RotSARJStop(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
{
if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,true,(char *)&ident)) > 0)
	{
	ves->clbkConsumeBufferedKey(OAPI_KEY_NUMPAD2,true,(char *)&dbl);
	InvalidateDisplay();
	}
}
}
}

void ITSAMFD::RotSARJNeg(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
{
if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,true,(char *)&ident)) > 0)
	{
	ves->clbkConsumeBufferedKey(OAPI_KEY_NUMPAD3,true,(char *)&dbl);
	InvalidateDisplay();
	}
}
}
}


void ITSAMFD::RotABPos(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
{
if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,true,(char *)&ident)) > 0)
	{
	ves->clbkConsumeBufferedKey(OAPI_KEY_NUMPAD4,true,(char *)&dbl);
	InvalidateDisplay();
	}
}
}
}

void ITSAMFD::RotABStop(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
{
if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,true,(char *)&ident)) > 0)
	{
	ves->clbkConsumeBufferedKey(OAPI_KEY_NUMPAD5,true,(char *)&dbl);
	InvalidateDisplay();
	}
}
}
}

void ITSAMFD::RotABNeg(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
{
if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,true,(char *)&ident)) > 0)
	{
	ves->clbkConsumeBufferedKey(OAPI_KEY_NUMPAD6,true,(char *)&dbl);
	InvalidateDisplay();
	}
}
}
}


void ITSAMFD::RotCDPos(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
{
if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,true,(char *)&ident)) > 0)
	{
	ves->clbkConsumeBufferedKey(OAPI_KEY_NUMPAD7,true,(char *)&dbl);
	InvalidateDisplay();
	}
}
}
}

void ITSAMFD::RotCDStop(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
{
if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,true,(char *)&ident)) > 0)
	{
	ves->clbkConsumeBufferedKey(OAPI_KEY_NUMPAD8,true,(char *)&dbl);
	InvalidateDisplay();
	}
}
}
}

void ITSAMFD::RotCDNeg(void)
{
double dbl=1;
MyVes = oapiGetVesselByName(vesname);
if(MyVes)
{
VESSEL2* ves = (VESSEL2*)oapiGetVesselInterface(MyVes);
int vv = ves->Version();
if (vv == 0);
else
{
if((ves->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,true,(char *)&ident)) > 0)
	{
	ves->clbkConsumeBufferedKey(OAPI_KEY_NUMPAD9,true,(char *)&dbl);
	InvalidateDisplay();
	}
}
}
}

void ITSAMFD::Update (HDC hDC)
{
	GetArray();

	SelectDefaultFont(hDC,0);
	HPEN gridpen = CreatePen(PS_SOLID,1,0x0033FF33);

	Title (hDC, "ITSArrayMFD");

	if (itsa_page == 0)
	{

		SelectDefaultFont(hDC,0);
		SetTextColor(hDC, 0x0033FF33);	// green


		TextOut(hDC,(20*cw),(ch*3),"Arrays", 6);

		TextOut(hDC,(20*cw),(ch*18),"Total Output", 12);

		SelectObject(hDC,gridpen);
		MoveToEx (hDC,(W/2), (0), NULL);
		LineTo (hDC,(W/2),(H));
		DeleteObject(gridpen);


		SetTextColor(hDC, 0x00999999);	// white

		TextOut(hDC,(0.5*cw),(ch*3),"Select Array", 12);

		TextOut(hDC,(0.5*cw),(ch*6),"Deploy/Pause/Stow", 17);
		TextOut(hDC,(0.5*cw),(ch*7),"Solar Panels", 12);

		TextOut(hDC,(0.5*cw),(ch*9.5),"Auto Sun Tracking", 17);
		TextOut(hDC,(0.5*cw),(ch*10.5),"On/Off", 6);

		TextOut(hDC,(0.5*cw),(ch*13.4),"Grapple Attachment", 18);

		TextOut(hDC,(0.5*cw),(ch*16.6),"Release Attachment", 18);

		TextOut(hDC,(0.5*cw),(ch*20),"Next Page", 9);


		SetTextColor(hDC, 0x0033FFFF);	// Yellow

		//////////// selected array ////////////////////
		{
		char kbuf[256];
		sprintf(kbuf,"%s", (vesname) );
		TextOut(hDC,(cw*0.5),(ch*4),kbuf,strlen(kbuf));
		}

		//////////////////// arrays    power ////////////////////
		if (strlen(vesname2[0])> 1){
		char kbuf[256];
		sprintf(kbuf,"%s", (vesname2[0]) );
		int stl = strlen(kbuf);
		if (stl > 9)stl=9;
		TextOut(hDC,(cw*20),(ch*4),kbuf,stl);
		sprintf(kbuf,"%0.3f kW", (ves_power[0]/1000) );
		stl = strlen(kbuf);
		TextOut(hDC,(W-(stl*cw)),(ch*4),kbuf,stl);
		}
			if (strlen(vesname2[1])> 1){
		char kbuf[256];
		sprintf(kbuf,"%s", (vesname2[1]) );
		int stl = strlen(kbuf);
		if (stl > 9)stl=9;
		TextOut(hDC,(cw*20),(ch*5),kbuf,stl);
		sprintf(kbuf,"%0.3f kW", (ves_power[1]/1000) );
		stl = strlen(kbuf);
		TextOut(hDC,(W-(stl*cw)),(ch*5),kbuf,stl);
		}
			if (strlen(vesname2[2])> 1){
		char kbuf[256];
		sprintf(kbuf,"%s", (vesname2[2]) );
		int stl = strlen(kbuf);
		if (stl > 9)stl=9;
		TextOut(hDC,(cw*20),(ch*6),kbuf,stl);
		sprintf(kbuf,"%0.3f kW", (ves_power[2]/1000) );
		stl = strlen(kbuf);
		TextOut(hDC,(W-(stl*cw)),(ch*6),kbuf,stl);
		}
			if (strlen(vesname2[3])> 1){
		char kbuf[256];
		sprintf(kbuf,"%s", (vesname2[3]) );
		int stl = strlen(kbuf);
		if (stl > 9)stl=9;
		TextOut(hDC,(cw*20),(ch*7),kbuf,stl);
		sprintf(kbuf,"%0.3f kW", (ves_power[3]/1000) );
		stl = strlen(kbuf);
		TextOut(hDC,(W-(stl*cw)),(ch*7),kbuf,stl);
		}

			if (strlen(vesname2[4])> 1){
		char kbuf[256];
		sprintf(kbuf,"%s", (vesname2[4]) );
		int stl = strlen(kbuf);
		if (stl > 9)stl=9;
		TextOut(hDC,(cw*20),(ch*8),kbuf,stl);
		sprintf(kbuf,"%0.3f kW", (ves_power[4]/1000) );
		stl = strlen(kbuf);
		TextOut(hDC,(W-(stl*cw)),(ch*8),kbuf,stl);
		}
			if (strlen(vesname2[5])> 1){
		char kbuf[256];
		sprintf(kbuf,"%s", (vesname2[5]) );
		int stl = strlen(kbuf);
		if (stl > 9)stl=9;
		TextOut(hDC,(cw*20),(ch*9),kbuf,stl);
		sprintf(kbuf,"%0.3f kW", (ves_power[5]/1000) );
		stl = strlen(kbuf);
		TextOut(hDC,(W-(stl*cw)),(ch*9),kbuf,stl);
		}
			if (strlen(vesname2[6])> 1){
		char kbuf[256];
		sprintf(kbuf,"%s", (vesname2[6]) );
		int stl = strlen(kbuf);
		if (stl > 9)stl=9;
		TextOut(hDC,(cw*20),(ch*10),kbuf,stl);
		sprintf(kbuf,"%0.3f kW", (ves_power[6]/1000) );
		stl = strlen(kbuf);
		TextOut(hDC,(W-(stl*cw)),(ch*10),kbuf,stl);
		}
			if (strlen(vesname2[7])> 1){
		char kbuf[256];
		sprintf(kbuf,"%s", (vesname2[7]) );
		int stl = strlen(kbuf);
		if (stl > 9)stl=9;
		TextOut(hDC,(cw*20),(ch*11),kbuf,stl);
		sprintf(kbuf,"%0.3f kW", (ves_power[7]/1000) );
		stl = strlen(kbuf);
		TextOut(hDC,(W-(stl*cw)),(ch*11),kbuf,stl);
		}
			if (strlen(vesname2[8])> 1){
		char kbuf[256];
		sprintf(kbuf,"%s", (vesname2[8]) );
		int stl = strlen(kbuf);
		if (stl > 9)stl=9;
		TextOut(hDC,(cw*20),(ch*12),kbuf,stl);
		sprintf(kbuf,"%0.3f kW", (ves_power[8]/1000) );
		stl = strlen(kbuf);
		TextOut(hDC,(W-(stl*cw)),(ch*12),kbuf,stl);
		}
			if (strlen(vesname2[9])> 1){
		char kbuf[256];
		sprintf(kbuf,"%s", (vesname2[9]) );
		int stl = strlen(kbuf);
		if (stl > 9)stl=9;
		TextOut(hDC,(cw*20),(ch*13),kbuf,stl);
		sprintf(kbuf,"%0.3f kW", (ves_power[9]/1000) );
		stl = strlen(kbuf);
		TextOut(hDC,(W-(stl*cw)),(ch*13),kbuf,stl);
		}

		tot_power = 0.0;
		for(int i=0;i<10;i++)
		{
		tot_power = tot_power+ves_power[i];
		}

		{
		char kbuf[256];
		sprintf(kbuf,"%0.3f kW", (tot_power/1000));
		TextOut(hDC,(cw*20),(ch*19),kbuf,strlen(kbuf));
		}

		//// array status
		if (dep_status == 1)
		{
		char kbuf[256];
		sprintf(kbuf,"Closed");
		TextOut(hDC,(cw*0.5),(ch*8),kbuf,strlen(kbuf));
		}
		if (dep_status == 2)
		{
		char kbuf[256];
		sprintf(kbuf,"Opening");
		TextOut(hDC,(cw*0.5),(ch*8),kbuf,strlen(kbuf));
		}
		if (dep_status == 3)
		{
		char kbuf[256];
		sprintf(kbuf,"Open");
		TextOut(hDC,(cw*0.5),(ch*8),kbuf,strlen(kbuf));
		}
		if (dep_status == 4)
		{
		char kbuf[256];
		sprintf(kbuf,"Closing");
		TextOut(hDC,(cw*0.5),(ch*8),kbuf,strlen(kbuf));
		}
		if (dep_status == 5)
		{
		char kbuf[256];
		sprintf(kbuf,"Opening Paused");
		TextOut(hDC,(cw*0.5),(ch*8),kbuf,strlen(kbuf));
		}
		if (dep_status == 6)
		{
		char kbuf[256];
		sprintf(kbuf,"Closing Paused");
		TextOut(hDC,(cw*0.5),(ch*8),kbuf,strlen(kbuf));
		}

		/////// auto sun tracking status
		if (auto_sun_status == 1)
		{
		char kbuf[256];
		sprintf(kbuf,"Off");
		TextOut(hDC,(cw*0.5),(ch*11.5),kbuf,strlen(kbuf));
		}
		if (auto_sun_status == 2)
		{
		char kbuf[256];
		sprintf(kbuf,"On");
		TextOut(hDC,(cw*0.5),(ch*11.5),kbuf,strlen(kbuf));
		}

		/////////// attach status
		if (att_status == 1)
		{
		char kbuf[256];
		sprintf(kbuf,"Released");
		TextOut(hDC,(cw*0.5),(ch*17.6),kbuf,strlen(kbuf));
		}
		if (att_status == 2)
		{
		char kbuf[256];
		sprintf(kbuf,"Attached");
		TextOut(hDC,(cw*0.5),(ch*14.4),kbuf,strlen(kbuf));
		}



	}

//////////////////////////// next page /////////////////////////////////

	if (itsa_page == 1)
	{

		SelectDefaultFont(hDC,0);
		//SetTextColor(hDC, 0x0033FF33);	// green


		//SetTextColor(hDC, 0x00FF6666);	// blue

		SetTextColor(hDC, 0x00999999);	// white
		TextOut(hDC,(0.5*cw),(ch*20),"Prev.Page", 9);

		if (auto_sun_status == 2)
		{
		SetTextColor(hDC, 0x00222299);	// red

		TextOut(hDC,(0.5*cw),(ch*3),"Rotate SARJ +", 13);

		TextOut(hDC,(0.5*cw),(ch*6),"Stop SARJ", 9);

		TextOut(hDC,(0.5*cw),(ch*9.5),"Rotate SARJ -", 13);

		TextOut(hDC,(22*cw),(ch*3),"Rotate AB BGA +", 15);

		TextOut(hDC,(26*cw),(ch*6),"Stop AB BGA", 11);

		TextOut(hDC,(22*cw),(ch*9.5),"Rotate AB BGA -", 15);

		TextOut(hDC,(22*cw),(ch*13.4),"Rotate CD BGA +", 15);

		TextOut(hDC,(26*cw),(ch*16.6),"Stop CD BGA", 11);

		TextOut(hDC,(22*cw),(ch*20),"Rotate CD BGA -", 15);

		SetTextColor(hDC, 0x0033FFFF);	// Yellow
		TextOut(hDC,(17*cw),(ch*2),"AUTO", 4);
		}

		else
		{
		SetTextColor(hDC, 0x00999999);	// white
		if (sarj_status == 2)SetTextColor(hDC, 0x0033FFFF);	// Yellow
		TextOut(hDC,(0.5*cw),(ch*3),"Rotate SARJ +", 13);

		SetTextColor(hDC, 0x00999999);	// white
		if (sarj_status == 1)SetTextColor(hDC, 0x0033FFFF);	// Yellow
		TextOut(hDC,(0.5*cw),(ch*6),"Stop SARJ", 9);

		SetTextColor(hDC, 0x00999999);	// white
		if (sarj_status == 3)SetTextColor(hDC, 0x0033FFFF);	// Yellow
		TextOut(hDC,(0.5*cw),(ch*9.5),"Rotate SARJ -", 13);

		SetTextColor(hDC, 0x00999999);	// white
		if (ab_bga_status == 2)SetTextColor(hDC, 0x0033FFFF);	// Yellow
		TextOut(hDC,(22*cw),(ch*3),"Rotate AB BGA +", 15);

		SetTextColor(hDC, 0x00999999);	// white
		if (ab_bga_status == 1)SetTextColor(hDC, 0x0033FFFF);	// Yellow
		TextOut(hDC,(26*cw),(ch*6),"Stop AB BGA", 11);

		SetTextColor(hDC, 0x00999999);	// white
		if (ab_bga_status == 3)SetTextColor(hDC, 0x0033FFFF);	// Yellow
		TextOut(hDC,(22*cw),(ch*9.5),"Rotate AB BGA -", 15);

		SetTextColor(hDC, 0x00999999);	// white
		if (cd_bga_status == 2)SetTextColor(hDC, 0x0033FFFF);	// Yellow
		TextOut(hDC,(22*cw),(ch*13.4),"Rotate CD BGA +", 15);

		SetTextColor(hDC, 0x00999999);	// white
		if (cd_bga_status == 1)SetTextColor(hDC, 0x0033FFFF);	// Yellow
		TextOut(hDC,(26*cw),(ch*16.6),"Stop CD BGA", 11);

		SetTextColor(hDC, 0x00999999);	// white
		if (cd_bga_status == 3)SetTextColor(hDC, 0x0033FFFF);	// Yellow
		TextOut(hDC,(22*cw),(ch*20),"Rotate CD BGA -", 15);
		}


	}
}







void ITSAMFD::WriteStatus (FILEHANDLE scn) const
{

	for (int i = 0; i < (int)strlen(vesname); i++)
	{
		if (vesname[i] == ' ')
		{
			vesname[i] = '^';
		}
	}
	char bbuf[256];
	sprintf (bbuf, "%s", vesname);
	oapiWriteScenario_string (scn, "ARRAY", bbuf);

	oapiWriteScenario_int (scn, "ITSPG", itsa_page);
}

void ITSAMFD::ReadStatus (FILEHANDLE scn)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "ARRAY", 5))
			sscanf (line+5, "%s", &vesname);
		else if (!strnicmp (line, "ITSPG", 5))
			sscanf (line+5, "%d", &itsa_page);
	}


	for (int i = 0; i < (int)strlen(vesname); i++)
	{
		if (vesname[i] == '^')
		{
			vesname[i] = ' ';
		}
	}



	MyVes = oapiGetVesselByName(vesname);

	if (MyVes)
	{
	VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(MyVes);
	int vv = v->Version();
	if (vv == 0);
	else
		{
		dep_status = v->clbkConsumeBufferedKey(COMM_SOLARRAY_IDENT,false,(char *)&ident);
		if(dep_status > 0)
			{
			auto_sun_status = v->clbkConsumeBufferedKey(COMM_AUTOSUN_STATUS,false,(char *)&ident);
			att_status = v->clbkConsumeBufferedKey(COMM_ATT_STATUS,false,(char *)&ident);
			sarj_status = v->clbkConsumeBufferedKey(COMM_SARJ_STATUS,false,(char *)&ident);
			ab_bga_status = v->clbkConsumeBufferedKey(COMM_ABBGA_STATUS,false,(char *)&ident);
			cd_bga_status = v->clbkConsumeBufferedKey(COMM_CDBGA_STATUS,false,(char *)&ident);
			}
		}
	}

}


void ITSAMFD::StoreStatus (void) const
{
	saveprm.page     = itsa_page;
	sprintf (saveprm.array_name, "%s", vesname);

}

void ITSAMFD::RecallStatus (void)
{
	itsa_page = saveprm.page;
	if(strlen(saveprm.array_name) > 0)sprintf (vesname, "%s", saveprm.array_name);
}

ITSAMFD::SavePrm ITSAMFD::saveprm = {0,0};