// ==============================================================
//                 ORBITER MODULE: p6
//
//
// p6.cpp
// Control module for p6 vessel clasS
// by RussH & BrianJ
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
//#include <cstdio>
#include <cmath>
//#include <string>
//#include <stdio.h>

// ==============================================================
// Some vessel parameters
// ==============================================================

// key comm keys
	const DWORD COMM_SEND_POWER = 2300;
	const DWORD COMM_SOLARRAY_IDENT = 2301; // vessel identified as array if returns true
	const DWORD COMM_AUTOSUN_STATUS = 2302; // returns auto sun tracking status on/off
	const DWORD COMM_SARJ_STATUS = 2303; // returns sarj rotation status
	const DWORD COMM_ABBGA_STATUS = 2304;	// returns AB BGA rotation status
	const DWORD COMM_CDBGA_STATUS = 2305;	// returns CD BGA rotation status
	const DWORD COMM_ATT_STATUS = 2306;	// returns parent attachment status




const double BGA_DEPLOY_SPEED = 0.02;
const double SOLP_DEPLOY_SPEED = 0.01; // Time Per Panel
const double RAD_DEPLOY_SPEED = 0.02;

const double SOLP_ROT_SPEED = 0.002; // BGA rotation
const double MAX_GRAPPLING_DIST = 0.3;

VECTOR3 normal_ab_ecl; // normal to panels in ecl frame
VECTOR3 normal_cd_ecl;


#define GREEN RGB(0, 255, 0)
#define YELLOW RGB(255, 255, 0)
#define RED RGB(255, 0, 0)
#define WHITE RGB(255, 255, 255)

static UINT Apanelgrp[84];
static UINT Bpanelgrp[84];

static UINT Cpanelgrp[84];
static UINT Dpanelgrp[84];

static UINT ABtrussgrp[21];
static UINT CDtrussgrp[21];




class p6: public VESSEL2 {
public:
	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } solp_status, solp2_status, motors_anim_status, rad_status;
	p6 (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel) {}
	~p6();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);


	void CalculatePower(void);
	void CheckChildAttach(void);
	void clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);

	private:
	void DefineAnimations (void);

	void RevertSolp (void);
	void RevertSolp2 (void);
	void RevertMotors (void);
	void RevertRad(void);
	void SolpSun (void);

	void AutoGrapple (void);
	void ReleaseGrapple(void);

	ANIMATIONCOMPONENT_HANDLE ASection[84], BSection[85], Aarmrot, Barmrot, ABmove[2], ABrot, rad[16], ABtrusssection[21];
	MGROUP_TRANSFORM *Apanel[84], *Bpanel[84], *ABtruss[21];

	ANIMATIONCOMPONENT_HANDLE CSection[84], DSection[85], Carmrot, Darmrot, CDmove[2], CDrot, CDtrusssection[21];
	MGROUP_TRANSFORM *Cpanel[84], *Dpanel[84], *CDtruss[21];


	MGROUP_TRANSFORM *ABhinge2anim, *ABhinge1anim, *ABmoveanim, *ABrotanim, *ABframeanim, *Aarmrotanim, *Apanel1, *Apanel2;
	MGROUP_TRANSFORM *Apaneltoparmanim, *Barmrotanim, *Bpanel1, *Bpanel2, *Bpaneltoparmanim, *ABpaneltopcapanim, *ABpaneltopcapcranim;

	MGROUP_TRANSFORM *CDhinge2anim, *CDhinge1anim, *CDmoveanim, *CDrotanim, *CDframeanim, *Carmrotanim, *Cpanel1, *Cpanel2;
	MGROUP_TRANSFORM *Cpaneltoparmanim, *Darmrotanim, *Dpanel1, *Dpanel2, *Dpaneltoparmanim, *CDpaneltopcapanim, *CDpaneltopcapcranim;

	MGROUP_TRANSFORM *Rad1anim, *Rad2anim, *Rad3anim, *Rad4anim, *Rad5anim, *Rad6anim, *Rad7anim, *Rad8anim;
	MGROUP_TRANSFORM *Radarm1anim, *Radarm2anim, *Radarm3anim, *Radarm4anim, *Radarm5anim, *Radarm6anim, *Radarm7anim, *Radarm8anim;




	ATTACHMENTHANDLE P6_attachment3;



	UINT anim_solp, AB_truss_anim, motors_anim, ABrot_anim;
	double  solp_proc, AB_truss_proc, motors_proc, ABrot_proc;

	UINT anim_solp2, CD_truss_anim, CDrot_anim;
	double  solp2_proc, CD_truss_proc, CDrot_proc;

	UINT rad_anim;
	double rad_proc;

	int rot_sw;
	int rot_sw2;
	int auto_sw;
	int deploy_status;

	double powerab;
	double powercd;
	int power_tot;

	double error_target_ab;
	double error_current_ab;
	double error_target_cd;
	double error_current_cd;

	int err_sw_ab;
	int err_sw_cd;


};



void p6::DefineAnimations (void)
{


	AB_truss_anim = CreateAnimation (0); // 3.deploy trussAB
	CD_truss_anim = CreateAnimation (0); // 3.deploy trussCD


	motors_anim = CreateAnimation (0);	// 2.deploy BGA's
	anim_solp = CreateAnimation (0);	// 3.deploy solar panels AB
	anim_solp2 = CreateAnimation (0);	// 3.deploy panels CD

	ABrot_anim = CreateAnimation (0);	// 4.rotate panel AB around BGA
	CDrot_anim = CreateAnimation (0);	// 4.rotate panel CD around BGA
	rad_anim = CreateAnimation (0);		// 3.deploy radiator



	//////// A Panel deploy  ////////////

	static UINT ABhinge2grp[1] = {26};
	ABhinge2anim = new MGROUP_ROTATE(0, ABhinge2grp, 1, _V(-1.245437, -0.134542, 3.998551 ), _V(1,0,0), (float)(115*RAD));
	AddAnimationComponent (motors_anim, 0.5, 0.75, ABhinge2anim);

	static UINT ABhinge1grp[1] = {25};
	ABhinge1anim = new MGROUP_ROTATE(0, ABhinge1grp, 1, _V(-1.245437, -0.70742, 3.998551), _V(1,0,0), (float)(115*RAD));
	ABmove[0] = AddAnimationComponent (motors_anim, 0.5, 0.75, ABhinge1anim);

	static UINT ABmovegrp[1] = {29};
	ABmoveanim = new MGROUP_ROTATE(0, ABmovegrp, 1, _V(-1.245457 ,0.425581 ,4.690571), _V(1,0,0), (float)(-115*RAD));
	ABmove[1] = AddAnimationComponent (motors_anim, 0.5, 0.75, ABmoveanim, ABmove[0]);

	static UINT ABrotgrp[1] = {363};
	ABrotanim = new MGROUP_ROTATE(0, ABrotgrp, 1, _V(-0.649, -1.560572, 4.691436), _V(0,1,0), float(2*PI) );
	ABrot = AddAnimationComponent (ABrot_anim, 0, 1, ABrotanim, ABmove[1]);

///////////////////// AB truss ////////////////////////

	ABtrussgrp[0] = 405;
	ABframeanim  = new MGROUP_TRANSLATE(0, &ABtrussgrp[0], 1, _V(0, -1, 0));
	ABtrusssection[0] = AddAnimationComponent (AB_truss_anim, 0.9756, 1, ABframeanim, ABrot);

	for(int i=1;i<21;i++)
	{
	int seq = i;
	int grpno = seq+384;

	double timestart = 0.9756-(seq*0.04878);
	double timeend = 0.9756-((seq-1)*0.04878);

	ABtrussgrp[seq]= grpno;
	ABtruss[seq]= new MGROUP_TRANSLATE(0, &ABtrussgrp[seq], 1, _V(0,-2, 0));
	ABtrusssection[seq] = AddAnimationComponent (AB_truss_anim, timestart, timeend, ABtruss[seq], ABtrusssection[seq-1]);

	}



	///////////////////// End AB truss ////////////////////////


	static UINT Aarmrotgrp[1] = {27};
	Aarmrotanim = new MGROUP_ROTATE(0, Aarmrotgrp, 1, _V(-0.649, -1.560572, 4.691436), _V(0,1,0), float(0.5*PI) );
	Aarmrot = AddAnimationComponent (motors_anim, 0.8, 0.95, Aarmrotanim, ABrot);

	static UINT Apanel1grp[1] = {33};
	Apanel1 = new MGROUP_ROTATE(0, Apanel1grp, 1, _V(-1.19137, -1.210363, 3.7567), _V(0,0,-1), float(0.5*PI) );
	ASection[0] = AddAnimationComponent (anim_solp, 0, 1, Apanel1, Aarmrot);

	static UINT Apanel2grp[1] = {34};
	Apanel2 = new MGROUP_ROTATE(0, Apanel2grp, 1, _V(-0.69137, -1.211363, 3.7567), _V(0,0,1), float(PI) );
	ASection[1] = AddAnimationComponent (anim_solp, 0, 1, Apanel2, ASection[0]);


	for(int i=1;i<41;i++)
	{
	int seq = 2*i;

	double posinc = (seq)*0.001;
	double ypos = -(1.210363+posinc);
	VECTOR3 posvect = _V(-1.19137, ypos, 3.7567);
	int grpno = seq+33;

	Apanelgrp[seq]= grpno;
	Apanel[seq]= new MGROUP_ROTATE(0, &Apanelgrp[seq], 1, posvect, _V(0,0,-1), float(PI) );
	ASection[seq] = AddAnimationComponent (anim_solp, 0, 1, Apanel[seq], ASection[seq-1]);

	seq=seq+1;

	posinc = (seq)*0.001;
	ypos = -(1.210363+posinc);
	posvect = _V(-0.69137, ypos, 3.7567);
	grpno = seq+33;

	Apanelgrp[seq]= grpno;
	Apanel[seq]= new MGROUP_ROTATE(0, &Apanelgrp[seq], 1, posvect, _V(0,0,1), float(PI) );
	ASection[seq] = AddAnimationComponent (anim_solp, 0, 1, Apanel[seq], ASection[seq-1]);

	}

	static UINT Apaneltoparm[1] = {31};
	Apaneltoparmanim = new MGROUP_ROTATE(0, Apaneltoparm, 1, _V(-1.19137, -1.292363, 3.75677), _V(0,0,-1), float(0.5*PI) );
	ASection[82] = AddAnimationComponent (anim_solp, 0, 1, Apaneltoparmanim, ASection[81]);




	/////////// B Panel deploy ////////////////

	static UINT Barmrotgrp[1] = {28};
	Barmrotanim = new MGROUP_ROTATE(0, Barmrotgrp, 1,_V(-0.649, -1.560572, 4.691436), _V(0,-1,0), float(0.5*PI) );
	Barmrot = AddAnimationComponent (motors_anim, 0.8, 0.95, Barmrotanim, ABrot);


	static UINT Bpanel1grp[1] = {115};
	Bpanel1 = new MGROUP_ROTATE(0, Bpanel1grp, 1, _V(-0.112372, -1.210363, 3.7567), _V(0,0,1), float(0.5*PI) );
	BSection[0] = AddAnimationComponent (anim_solp, 0, 1, Bpanel1, Barmrot);

	static UINT Bpanel2grp[1] = {116};
	Bpanel2 = new MGROUP_ROTATE(0, Bpanel2grp, 1, _V(-0.612372, -1.211363, 3.7567 ), _V(0,0,-1), float(PI) );
	BSection[1] = AddAnimationComponent (anim_solp, 0, 1, Bpanel2, BSection[0]);


	for(int i=1;i<41;i++)
	{
	int seq = 2*i;

	double posinc = (seq)*0.001;
	double ypos = -(1.210363+posinc);
	VECTOR3 posvect = _V(-0.112372, ypos, 3.7567);
	int grpno = seq+115;

	Bpanelgrp[seq]= grpno;
	Bpanel[seq]= new MGROUP_ROTATE(0, &Bpanelgrp[seq], 1, posvect, _V(0,0,1), float(PI) );
	BSection[seq] = AddAnimationComponent (anim_solp, 0, 1, Bpanel[seq], BSection[seq-1]);

	seq=seq+1;

	posinc = (seq)*0.001;
	ypos = -(1.210363+posinc);
	posvect = _V(-0.612372, ypos, 3.7567);
	grpno = seq+115;

	Bpanelgrp[seq]= grpno;
	Bpanel[seq]= new MGROUP_ROTATE(0, &Bpanelgrp[seq], 1, posvect, _V(0,0,-1), float(PI) );
	BSection[seq] = AddAnimationComponent (anim_solp, 0, 1, Bpanel[seq], BSection[seq-1]);

	}

	static UINT Bpaneltoparm[1] = {30};
	Bpaneltoparmanim = new MGROUP_ROTATE(0, Bpaneltoparm, 1, _V(-0.112372, -1.292363, 3.75677), _V(0,0,1), float(0.5*PI) );
	BSection[82] = AddAnimationComponent (anim_solp, 0, 1, Bpaneltoparmanim, BSection[81]);

	static UINT ABpaneltopcap[1] = {};
	ABpaneltopcapanim = new MGROUP_ROTATE(0, ABpaneltopcap, 0, _V(-0.112372, -1.292363, 3.75677), _V(0,0,1), float(0.5*PI) );
	BSection[83] = AddAnimationComponent (anim_solp, 0, 1, ABpaneltopcapanim, BSection[81]);

	static UINT ABpaneltopcapcr[1] = {32};
	ABpaneltopcapcranim = new MGROUP_ROTATE(0, ABpaneltopcapcr, 1, _V(-0.649, -1.560572, 4.691436), _V(0,1,0), float(0.5*PI) );
	BSection[84] = AddAnimationComponent (motors_anim, 0.8, 0.95, ABpaneltopcapcranim, BSection[83]);



	/////// C panel deploy /////////

	static UINT CDhinge2grp[1] = {19};
	CDhinge2anim = new MGROUP_ROTATE(0, CDhinge2grp, 1, _V(0, 0.399882, 3.998551), _V(-1,0,0), (float)(115*RAD));
	AddAnimationComponent (motors_anim, 0.0, 0.25, CDhinge2anim);

	static UINT CDhinge1grp[1] = {18};
	CDhinge1anim  = new MGROUP_ROTATE(0, CDhinge1grp, 1, _V(0 ,0.97276, 3.998551), _V(-1,0,0), (float)(115*RAD));
	CDmove[0] = AddAnimationComponent (motors_anim, 0.0, 0.25, CDhinge1anim);

	static UINT CDmovegrp[1] = {361};
	CDmoveanim  = new MGROUP_ROTATE(0, CDmovegrp, 1, _V(0, -0.154175, 4.690571), _V(1,0,0), (float)(115*RAD));
	CDmove[1] = AddAnimationComponent (motors_anim, 0.0, 0.25, CDmoveanim, CDmove[0]);

	static UINT CDrotgrp[1] = {362};
	CDrotanim  = new MGROUP_ROTATE(0, CDrotgrp, 1, _V(0.65036, 1.6195, 4.689622), _V(0,1,0), float(2*PI) );
	CDrot = AddAnimationComponent (CDrot_anim, 0, 1, CDrotanim, CDmove[1]);


	///////////////////// CD truss ////////////////////////

	CDtrussgrp[0] = 384;
	CDframeanim  = new MGROUP_TRANSLATE(0, &CDtrussgrp[0], 1, _V(0, 1, 0));
	CDtrusssection[0] = AddAnimationComponent (CD_truss_anim, 0.9756, 1, CDframeanim, CDrot);

	for(int i=1;i<21;i++)
	{
	int seq = i;
	int grpno = seq+363;

	double timestart = 0.9756-(seq*0.04878);
	double timeend = 0.9756-((seq-1)*0.04878);

	CDtrussgrp[seq]= grpno;
	CDtruss[seq]= new MGROUP_TRANSLATE(0, &CDtrussgrp[seq], 1, _V(0, 2, 0));
	CDtrusssection[seq] = AddAnimationComponent (CD_truss_anim, timestart, timeend, CDtruss[seq], CDtrusssection[seq-1]);

	}



	///////////////////// End CD truss ////////////////////////


	static UINT Carmrotgrp[1] = {20};
	Carmrotanim = new MGROUP_ROTATE(0, Carmrotgrp, 1, _V(0.65036, 1.6195, 4.689622), _V(0,-1,0), float(0.5*PI) );
	Carmrot = AddAnimationComponent (motors_anim, 0.3, 0.45, Carmrotanim, CDrot);

	static UINT Cpanel1grp[1] = {197};
	Cpanel1 = new MGROUP_ROTATE(0, Cpanel1grp, 1, _V(1.19137, 1.475704, 3.7567), _V(0,0,-1), (float)(0.5*PI));
	CSection[0] = AddAnimationComponent (anim_solp2, 0, 1, Cpanel1, Carmrot);

	static UINT Cpanel2grp[1] = {198};
	Cpanel2 = new MGROUP_ROTATE(0, Cpanel2grp, 1, _V(0.69137, 1.476704, 3.7567), _V(0,0,1), float(PI) );
	CSection[1] = AddAnimationComponent (anim_solp2, 0, 1, Cpanel2, CSection[0]);


	for(int i=1;i<41;i++)
	{
	int seq = 2*i;

	double posinc = (seq)*0.001;
	double ypos = (1.475704+posinc);
	VECTOR3 posvect = _V(1.19137, ypos, 3.7567);
	int grpno = seq+197;

	Cpanelgrp[seq]= grpno;
	Cpanel[seq]= new MGROUP_ROTATE(0, &Cpanelgrp[seq], 1, posvect, _V(0,0,-1), float(PI) );
	CSection[seq] = AddAnimationComponent (anim_solp2, 0, 1, Cpanel[seq], CSection[seq-1]);

	seq=seq+1;

	posinc = (seq)*0.001;
	ypos = (1.475704+posinc);
	posvect = _V(0.69137, ypos, 3.7567);
	grpno = seq+197;

	Cpanelgrp[seq]= grpno;
	Cpanel[seq]= new MGROUP_ROTATE(0, &Cpanelgrp[seq], 1, posvect, _V(0,0,1), float(PI) );
	CSection[seq] = AddAnimationComponent (anim_solp2, 0, 1, Cpanel[seq], CSection[seq-1]);

	}

	static UINT Cpaneltoparm[1] = {23};
	Cpaneltoparmanim = new MGROUP_ROTATE(0, Cpaneltoparm, 1, _V(1.19137, 1.557703, 3.7567), _V(0,0,-1), float(0.5*PI) );
	CSection[82] = AddAnimationComponent (anim_solp2, 0, 1, Cpaneltoparmanim, CSection[81]);




	/////////// D Panel deploy ////////////////

	static UINT Darmrotgrp[1] = {21};
	Darmrotanim = new MGROUP_ROTATE(0, Darmrotgrp, 1, _V(0.65036, 1.6195, 4.689622), _V(0,1,0), float(0.5*PI) );
	Darmrot = AddAnimationComponent (motors_anim, 0.3, 0.45, Darmrotanim, CDrot);


	static UINT Dpanel1grp[1] = {279};
	Dpanel1 = new MGROUP_ROTATE(0, Dpanel1grp, 1, _V(0.112372, 1.475704, 3.7567), _V(0,0,1), float(0.5*PI) );
	DSection[0] = AddAnimationComponent (anim_solp2, 0, 1, Dpanel1, Darmrot);

	static UINT Dpanel2grp[1] = {280};
	Dpanel2 = new MGROUP_ROTATE(0, Dpanel2grp, 1, _V(0.612372, 1.476704, 3.7567), _V(0,0,-1), float(PI) );
	DSection[1] = AddAnimationComponent (anim_solp2, 0, 1, Dpanel2, DSection[0]);


	for(int i=1;i<41;i++)
	{
	int seq = 2*i;

	double posinc = (seq)*0.001;
	double ypos = (1.475704+posinc);
	VECTOR3 posvect = _V(0.112372, ypos, 3.7567);
	int grpno = seq+279;

	Dpanelgrp[seq]= grpno;
	Dpanel[seq]= new MGROUP_ROTATE(0, &Dpanelgrp[seq], 1, posvect, _V(0,0,1), float(PI) );
	DSection[seq] = AddAnimationComponent (anim_solp2, 0, 1, Dpanel[seq], DSection[seq-1]);

	seq=seq+1;

	posinc = (seq)*0.001;
	ypos = (1.475704+posinc);
	posvect = _V(0.612372, ypos, 3.7567);
	grpno = seq+279;

	Dpanelgrp[seq]= grpno;
	Dpanel[seq]= new MGROUP_ROTATE(0, &Dpanelgrp[seq], 1, posvect, _V(0,0,-1), float(PI) );
	DSection[seq] = AddAnimationComponent (anim_solp2, 0, 1, Dpanel[seq], DSection[seq-1]);

	}

	static UINT Dpaneltoparm[1] = {22};
	Dpaneltoparmanim = new MGROUP_ROTATE(0, Dpaneltoparm, 1, _V(0.112372, 1.557703, 3.7567), _V(0,0,1), float(0.5*PI) );
	DSection[82] = AddAnimationComponent (anim_solp2, 0, 1, Dpaneltoparmanim, DSection[81]);

	static UINT CDpaneltopcap[1] = {};
	CDpaneltopcapanim = new MGROUP_ROTATE(0, CDpaneltopcap, 0, _V(0.112372, 1.557703, 3.7567), _V(0,0,1), float(0.5*PI) );
	DSection[83] = AddAnimationComponent (anim_solp2, 0, 1, CDpaneltopcapanim, DSection[81]);

	static UINT CDpaneltopcapcr[1] = {24};
	CDpaneltopcapcranim = new MGROUP_ROTATE(0, CDpaneltopcapcr, 1, _V(0.65036, 1.6195, 4.689622), _V(0,-1,0), float(0.5*PI) );
	DSection[84] = AddAnimationComponent (motors_anim, 0.3, 0.45, CDpaneltopcapcranim, DSection[83]);


	/////////////// radiator /////////////////


	static UINT Radiator00Grp[1] = {0};
	Rad1anim = new MGROUP_ROTATE(0, Radiator00Grp, 1, _V(1.43527, 0.55088, 2.9281), _V(0,0,-1), (float)(90*RAD));//80
	rad[0] = AddAnimationComponent (rad_anim, 0, 1, Rad1anim);

	static UINT Radiator01Grp[1] = {1};
	Rad2anim = new MGROUP_ROTATE(0, Radiator01Grp, 1, _V(1.48078, 1.25162, 2.9364), _V(0,0,1), (float)(165*RAD));//160
	rad[1] = AddAnimationComponent (rad_anim, 0, 1, Rad2anim, rad[0]);

	static UINT Radiator02Grp[1] = {2};
	Rad3anim = new MGROUP_ROTATE(0, Radiator02Grp, 1, _V(1.52074, -0.51763, 2.9364), _V(0,0,-1), (float)(150*RAD));//160
	rad[2] = AddAnimationComponent (rad_anim, 0, 1, Rad3anim, rad[1]);

	static UINT Radiator03Grp[1] = {3};
	Rad4anim = new MGROUP_ROTATE(0, Radiator03Grp, 1, _V(1.55592, 1.25166, 2.9438), _V(0,0,1), (float)(150*RAD));  //160
	rad[3] = AddAnimationComponent (rad_anim, 0, 1, Rad4anim, rad[2]);

	static UINT Radiator04Grp[1] = {4};
	Rad5anim = new MGROUP_ROTATE(0, Radiator04Grp, 1, _V(1.59638, -0.51766, 2.9438 ), _V(0,0,-1), (float)(150*RAD));//160
	rad[4] = AddAnimationComponent (rad_anim, 0, 1, Rad5anim, rad[3]);

	static UINT Radiator05Grp[1] = {5};
	Rad6anim = new MGROUP_ROTATE(0, Radiator05Grp, 1, _V(1.63662, 1.25156, 2.9438 ), _V(0,0,1), (float)(150*RAD));//160
	rad[5] = AddAnimationComponent (rad_anim, 0, 1, Rad6anim, rad[4]);

	static UINT Radiator06Grp[1] = {6};
	Rad7anim = new MGROUP_ROTATE(0, Radiator06Grp, 1, _V(1.68244, -0.51763, 2.9364 ), _V(0,0,-1), (float)(150*RAD));//160
	rad[6] = AddAnimationComponent (rad_anim, 0, 1, Rad7anim, rad[5]);

	static UINT Radiator07Grp[1] = {7};
	Rad8anim = new MGROUP_ROTATE(0, Radiator07Grp, 1, _V(1.72338, 1.25182, 2.9364), _V(0,0,1), (float)(150*RAD));//160
	AddAnimationComponent (rad_anim, 0, 1, Rad8anim, rad[6]);


	///////////// radiator arms ///////////////////////

	static UINT Radiator_a00Grp[1] = {17};
	Radarm1anim = new MGROUP_ROTATE(0, Radiator_a00Grp, 1, _V(1.4938, 0.086, 2.9595), _V(0,0,1), (float)(90*RAD));
	rad[7] = AddAnimationComponent (rad_anim, 0, 1, Radarm1anim);

	static UINT Radiator_a01Grp[1] = {16};
	Radarm2anim = new MGROUP_ROTATE(0, Radiator_a01Grp, 1, _V(1.5188, -0.41499, 2.9594), _V(0,0,1), (float)(-165*RAD));//170
	rad[8] = AddAnimationComponent (rad_anim, 0, 1, Radarm2anim, rad[7]);

	static UINT Radiator_a02Grp[1] = {15};
	Radarm3anim = new MGROUP_ROTATE(0, Radiator_a02Grp, 1, _V(1.5438, 1.36499, 2.9595), _V(0,0,1), (float)(150*RAD)); //160
	rad[9] = AddAnimationComponent (rad_anim, 0, 1, Radarm3anim, rad[8]);

	static UINT Radiator_a03Grp[1] = {14};
	Radarm4anim = new MGROUP_ROTATE(0, Radiator_a03Grp, 1, _V(1.56882, -0.41499, 2.9596), _V(0,0,1), (float)(-150*RAD));//160
	rad[10] = AddAnimationComponent (rad_anim, 0, 1, Radarm4anim, rad[9]);

	static UINT Radiator_a04Grp[1] = {13};
	Radarm5anim = new MGROUP_ROTATE(0, Radiator_a04Grp, 1, _V(1.58878, 1.36499, 2.9596), _V(0,0,1), (float)(150*RAD));//160
	rad[11] = AddAnimationComponent (rad_anim, 0, 1, Radarm5anim, rad[10]);

	static UINT Radiator_a05Grp[1] = {12};
	Radarm6anim = new MGROUP_ROTATE(0, Radiator_a05Grp, 1, _V(1.61882, -0.41499, 2.9396), _V(0,0,1), (float)(-150*RAD));//160
	rad[12] = AddAnimationComponent (rad_anim, 0, 1, Radarm6anim, rad[11]);

	static UINT Radiator_a06Grp[1] = {11};
	Radarm7anim = new MGROUP_ROTATE(0, Radiator_a06Grp, 1, _V(1.6438, 1.36499, 2.9596), _V(0,0,1), (float)(150*RAD));//160
	rad[13] = AddAnimationComponent (rad_anim, 0, 1, Radarm7anim, rad[12]);

	static UINT Radiator_a07Grp[1] = {10};
	Radarm8anim = new MGROUP_ROTATE(0, Radiator_a07Grp, 1, _V(1.66881, -0.415, 2.9596), _V(0,0,1), (float)(-150*RAD));//170
	AddAnimationComponent (rad_anim, 0, 1, Radarm8anim, rad[13]);


}

// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
p6::~p6 ()
{

	delete ABhinge2anim;
	delete ABhinge1anim;
	delete ABmoveanim;
	delete ABrotanim;
	delete ABframeanim;
	delete Aarmrotanim;
	delete Apanel1;
	delete Apanel2;
	delete Apaneltoparmanim;
	delete Barmrotanim;
	delete Bpanel1;
	delete Bpanel2;
	delete Bpaneltoparmanim;
	delete ABpaneltopcapanim;
	delete ABpaneltopcapcranim;

	delete CDhinge2anim;
	delete CDhinge1anim;
	delete CDmoveanim;
	delete CDrotanim;
	delete CDframeanim;
	delete Carmrotanim;
	delete Cpanel1;
	delete Cpanel2;
	delete Cpaneltoparmanim;
	delete Darmrotanim;
	delete Dpanel1;
	delete Dpanel2;
	delete Dpaneltoparmanim;
	delete CDpaneltopcapanim;
	delete CDpaneltopcapcranim;

	delete Rad1anim;
	delete Rad2anim;
	delete Rad3anim;
	delete Rad4anim;
	delete Rad5anim;
	delete Rad6anim;
	delete Rad7anim;
	delete Rad8anim;
	delete Radarm1anim;
	delete Radarm2anim;
	delete Radarm3anim;
	delete Radarm4anim;
	delete Radarm5anim;
	delete Radarm6anim;
	delete Radarm7anim;
	delete Radarm8anim;


	int i;
	for (i = 0; i < 85; i++)
	{
		delete Apanel[i];
		delete Bpanel[i];
		delete Cpanel[i];
		delete Dpanel[i];
	}

	for (i = 0; i < 21; i++)
	{
		delete ABtruss[i];
		delete CDtruss[i];

	}
}

void p6::RevertSolp (void)
{
	solp_status = ((solp_status == DOOR_CLOSED || solp_status == DOOR_CLOSING) ?
		DOOR_OPENING : DOOR_CLOSING);
}

void p6::RevertSolp2 (void)
{
	solp2_status = ((solp2_status == DOOR_CLOSED || solp2_status == DOOR_CLOSING) ?
		DOOR_OPENING : DOOR_CLOSING);
}

void p6::RevertMotors (void)
{
	motors_anim_status = ((motors_anim_status == DOOR_CLOSED || motors_anim_status == DOOR_CLOSING) ?
		DOOR_OPENING : DOOR_CLOSING);
}

void p6::RevertRad (void)
{
	rad_status = ((rad_status == DOOR_CLOSED || rad_status == DOOR_CLOSING) ?
		DOOR_OPENING : DOOR_CLOSING);
}
// ==============================================================
// p6 class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void p6::clbkSetClassCaps (FILEHANDLE cfg)
{

	rot_sw = 0;
	rot_sw2 = 0;
	auto_sw = 0;
	deploy_status = 0;


	solp_proc = 0.0;
	solp2_proc = 0.0;
	motors_proc = 0.0;
	AB_truss_proc = 0.0;
	CD_truss_proc = 0.0;
	rad_proc = 0.0;

	solp_status = DOOR_CLOSED;
	solp2_status = DOOR_CLOSED;
	motors_anim_status = DOOR_CLOSED;
	rad_status = DOOR_CLOSED;

	DefineAnimations();

	error_current_ab = (1 - 0.002*(oapiRand ()));
	err_sw_ab = 0;
	error_current_cd = (1 - 0.002*(oapiRand ()));
	err_sw_cd = 0;


	// physical specs
	SetCW (0.3, 0.4, 0.6, 0.6);
	SetRotDrag (_V(1,1,1));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-3);
		SetBankMomentScale (1e-3);
	}
	SetTrimScale (0.1);
	//SetEmptyMass(15838);
	SetMeshVisibilityMode ((AddMesh ("MBR_ITS/mbr_its6")), MESHVIS_ALWAYS);
	//SetCameraOffset (_V(0,3,-5));
	//SetSize (17.0);
	SetPMI (_V(9, 9, 2));
	SetCrossSections (_V(26, 30, 10));
	SetSurfaceFrictionCoeff (0.5, 0.5);
	SetTouchdownPoints (_V(0,-1,9), _V(-2,-1,-8), _V(2,-1,-8));
	//EnableTransponder (true);

	//DOCKHANDLE Dock_0 = CreateDock (_V(0, 0.420657, -6.3), _V(0,0,-1), _V(0,1,0));

	//P3_attachment0  = CreateAttachment (true,_V(1.130798, 1.670001, -4.023128), _V(0.859581, 0.511, 0.0), _V(-0.511, 0.86,0.0),"GS");
	//P3_attachment1  = CreateAttachment (true,_V(-1.130798, 1.670001, -4.023128), _V(-0.859581, 0.511, 0.0), _V(0.511, 0.86,0.0),"GS");
   // P3_attachment2  = CreateAttachment (true,_V(0, 0, 0),_V(0, -1, 0),_V(0,0,-1),"XS");

	P6_attachment3  = CreateAttachment (false,_V(-1.46, -0.8, 4.051),_V(0, 0, 1),_V(0,1,0),"MP");

	// reentry pstream
	PARTICLESTREAMSPEC rps = {
		0, 10, 40, 0, 0.001, 0.5, 4, 0.2, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 1, PARTICLESTREAMSPEC::ATM_PLOG, 6e7, 5e15
	};
	AddReentryStream (&rps);


}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new p6 (hvessel, flightmodel);
}
// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (p6*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void p6::clbkPostCreation ()
{
	SetAnimation (motors_anim, motors_proc);
	SetAnimation (AB_truss_anim, AB_truss_proc);
	SetAnimation (anim_solp, solp_proc);
	SetAnimation (ABrot_anim, ABrot_proc);
	SetAnimation (CD_truss_anim, CD_truss_proc);
	SetAnimation (anim_solp2, solp2_proc);
	SetAnimation (CDrot_anim, CDrot_proc);
	SetAnimation (rad_anim, rad_proc);
}
void p6::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL2::clbkSaveState (scn);

	// custom parameters

	oapiWriteScenario_int (scn, "AUTOSUN", auto_sw);

	oapiWriteScenario_int (scn, "DEPLOY", deploy_status);

	sprintf (cbuf, "%d %0.4f", motors_anim_status, motors_proc);
	oapiWriteScenario_string (scn, "MOTORS", cbuf);

	sprintf (cbuf, "%d %0.4f %0.4f", solp_status, solp_proc, AB_truss_proc);
	oapiWriteScenario_string (scn, "SOLPAB", cbuf);

	sprintf (cbuf, "%d %0.4f %0.4f", solp2_status, solp2_proc, CD_truss_proc);
	oapiWriteScenario_string (scn, "SOLPCD", cbuf);

	sprintf (cbuf, "%d %0.4f", rad_status, rad_proc);
	oapiWriteScenario_string (scn, "RAD", cbuf);

	sprintf (cbuf, "%d %0.4f %d %0.4f", rot_sw, ABrot_proc, rot_sw2, CDrot_proc);
	oapiWriteScenario_string (scn, "SOLPROT", cbuf);

}
void p6::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
        if (!strnicmp (line, "AUTOSUN", 7)) {
            sscanf (line+7, "%d", &auto_sw);
		} else if (!strnicmp (line, "DEPLOY", 6)) {
			sscanf (line+6, "%d", &deploy_status);
		} else if (!strnicmp (line, "MOTORS", 6)) {
			sscanf (line+6, "%d%lf", &motors_anim_status, &motors_proc);
		} else if (!strnicmp (line, "SOLPAB", 6)) {
			sscanf (line+6, "%d%lf%lf", &solp_status, &solp_proc, &AB_truss_proc);
		} else if (!strnicmp (line, "SOLPCD", 6)) {
			sscanf (line+6, "%d%lf%lf", &solp2_status, &solp2_proc, &CD_truss_proc);
		} else if (!strnicmp (line, "RAD", 3)) {
			sscanf (line+3, "%d%lf", &rad_status, &rad_proc);
		} else if (!strnicmp (line, "SOLPROT", 7)) {
			sscanf (line+7, "%d%lf%d%lf", &rot_sw, &ABrot_proc, &rot_sw2, &CDrot_proc);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }


	SetAnimation (motors_anim, motors_proc);
	SetAnimation (AB_truss_anim, AB_truss_proc);
	SetAnimation (anim_solp, solp_proc);
	SetAnimation (ABrot_anim, ABrot_proc);
	SetAnimation (CD_truss_anim, CD_truss_proc);
	SetAnimation (anim_solp2, solp2_proc);
	SetAnimation (CDrot_anim, CDrot_proc);
	SetAnimation (rad_anim, rad_proc);

}
int p6::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
	///////////////key comm stuff///////

	if (key == COMM_SEND_POWER) {
			return power_tot;
       }
	if (key == COMM_SOLARRAY_IDENT) {
			return (deploy_status+1);
       }
	if (key == COMM_AUTOSUN_STATUS) {
			return (auto_sw+1);
		}

	if (key == COMM_ABBGA_STATUS) {
			return (rot_sw+1);
		}
	if (key == COMM_CDBGA_STATUS) {
			return (rot_sw2+1);
		}
	if (key == COMM_ATT_STATUS) {
		if (GetAttachmentStatus (P6_attachment3))return 2;
		else return 1;
		}


	///////////////normal key handling stuff

  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) {

  } else { // unmodified keys
    switch (key) {


		case OAPI_KEY_G: // deploy motors(BGA's), solar panels AB & CD, radiator
		if (deploy_status == 1)deploy_status = 4; // paused
		else if (deploy_status == 4)deploy_status = 1; // resume opening

		if (deploy_status == 3)deploy_status = 5; // paused
		else if (deploy_status == 5)deploy_status = 3; // resume closing

		if (deploy_status == 0) // closed
		{
			deploy_status = 1;  // opening
			RevertMotors();
		}
		if (deploy_status == 2) // open
		{
			deploy_status = 3;  // closing
			auto_sw = 0;
			rot_sw = 0;
			rot_sw2 = 0;
		}
		return (deploy_status+1);

		case OAPI_KEY_K: // auto sun tracking on off
		if (solp2_status == DOOR_OPEN && deploy_status < 3)
		{
		if(auto_sw == 0)auto_sw = 1;
		else if(auto_sw == 1)
		{
			auto_sw = 0;
			rot_sw = 0;
			rot_sw2 = 0;
		}
		}
		return (auto_sw+1);

		case OAPI_KEY_N: // grapple
		AutoGrapple();
		return 1;

		case OAPI_KEY_M: // release attachment
		ReleaseGrapple();
		return 1;


		case OAPI_KEY_NUMPAD7: // CD panels rot_ccw
		if(solp2_status == DOOR_OPEN && deploy_status < 3)rot_sw2 = 1;
		return 1;

		case OAPI_KEY_NUMPAD9: // CD panels rot_cw
		if(solp2_status == DOOR_OPEN && deploy_status < 3)rot_sw2 = 2;
		return 1;

		case OAPI_KEY_NUMPAD8: // CD panels rot_stop
		if(solp2_status == DOOR_OPEN && deploy_status < 3)rot_sw2 = 0;
		return 1;

		case OAPI_KEY_NUMPAD4: // AB panels rot_ccw
		if(solp2_status == DOOR_OPEN && deploy_status < 3)rot_sw = 1;
		return 1;

		case OAPI_KEY_NUMPAD6: // AB panels rot_cw
		if(solp2_status == DOOR_OPEN && deploy_status < 3)rot_sw = 2;
		return 1;

		case OAPI_KEY_NUMPAD5: // AB panels rot_stop
		if(solp2_status == DOOR_OPEN && deploy_status < 3)rot_sw = 0;
		DeactivateNavmode(NAVMODE_KILLROT);
		return 1;

	}
  }
  return 0;
}


void p6::clbkPreStep(double SimT, double SimDT, double MJD) {

	if (auto_sw == 1) SolpSun(); // begin auto sun tracking
	if (deploy_status == 1)	// opening solar panels
	{
	if (solp2_status == 0 && motors_anim_status == 1) RevertSolp2(); // deploy solar panels AB after BGA's deployed
	if (solp_status == 0 && solp2_status == 1) RevertSolp(); // deploy solar panels CD after AB deployed
	if (rad_status == 0 && solp_status == 1) RevertRad(); // deploy rad after panels CD deployed
	}
	if (deploy_status == 3)		// closing solar panels
	{
	if (CDrot_proc == 0.0 && ABrot_proc == 0.0 && rad_status == 1) RevertRad();//clse rad after panels aligned
	if (solp2_status == 1 && rad_status == 0) RevertSolp2(); // close AB after rad closed
	if (solp_status == 1 && solp2_status == 0) RevertSolp(); // close CD after AB closed
	if (motors_anim_status == 1 && solp_status == 0) RevertMotors(); // stow BGAs
	}


	if (GetAttachmentStatus(P6_attachment3)) CheckChildAttach();
}




void p6::SolpSun (void) {				// auto sun tracking

	double adj = oapiGetTimeAcceleration(); // dead-band adjustment factor


	double Sun_angle;
	char cbuf[7]= "Sun";
	OBJHANDLE hRef = oapiGetObjectByName(cbuf);
	VECTOR3 rsunloc;
	VECTOR3 sunglobal;
	oapiGetGlobalPos (hRef, &sunglobal);	// get the global pos of Sun
	Global2Local(sunglobal,rsunloc);		// convert to vessel frame coords


	Sun_angle = atan2(rsunloc.x, rsunloc.z);	// current angle of sun around BGA rotation axis(- adjusted 180deg)
	if (Sun_angle < 0)Sun_angle = Sun_angle+(2*PI);
	else if (Sun_angle > (2*PI))Sun_angle = Sun_angle-(2*PI);


		/////////////// AB y axis rotation ////////////

{
		double Solp_angleb;

		// current BGA rotation angle
		Solp_angleb = 2*PI*(ABrot_proc);


	{
	if ((Solp_angleb - Sun_angle) > -0.001*adj && (Solp_angleb - Sun_angle) < 0.001*adj) //deadband
	{
		rot_sw = 0;								// turn off manual rotation
		ABrot_proc = Sun_angle/(2*PI);			// lock on to sun
		SetAnimation (ABrot_anim, ABrot_proc);	// apply to animation
	}

	// logic for deciding which way to turn to find sun
	else if ((Solp_angleb - Sun_angle) > 0)
	{
		if (Solp_angleb - Sun_angle < PI) rot_sw = 2;
		if (Solp_angleb - Sun_angle > PI) rot_sw = 1;
	}

	else if ((Solp_angleb - Sun_angle) < 0)
	{
		if (Solp_angleb - Sun_angle > -PI) rot_sw = 1;
		if (Solp_angleb - Sun_angle < -PI) rot_sw = 2;
	}
	}
}

		/////////////// CD y axis rotation ////////////

{
		double Solp_anglec;


		// current BGA rotation angle
		Solp_anglec = 2*PI*(CDrot_proc);


	{
	if ((Solp_anglec - Sun_angle) > -0.0005*adj && (Solp_anglec - Sun_angle) < 0.0005*adj) //deadband
	{
		rot_sw2 = 0;								// turn off manual rotation
		CDrot_proc = Sun_angle/(2*PI);			// lock on to sun
		SetAnimation (CDrot_anim, CDrot_proc);	// apply to animation
	}

	// logic for deciding which way to turn to find sun
	else if ((Solp_anglec - Sun_angle) > 0)
	{
		if (Solp_anglec - Sun_angle < PI) rot_sw2 = 2;
		if (Solp_anglec - Sun_angle > PI) rot_sw2 = 1;
	}

	else if ((Solp_anglec - Sun_angle) < 0)
	{
		if (Solp_anglec - Sun_angle > -PI) rot_sw2 = 1;
		if (Solp_anglec - Sun_angle < -PI) rot_sw2 = 2;
	}
	}
}

}

void p6::CalculatePower(void)
{

	char cbuf[7]= "Sun";
	OBJHANDLE hRef = oapiGetObjectByName(cbuf);
	VECTOR3 rsunloc;
	VECTOR3 sunglobal;
	oapiGetGlobalPos (hRef, &sunglobal);	// get the global pos of Sun
	Global2Local(sunglobal,rsunloc);		// convert to vessel frame coords
	double veclength = length(rsunloc);
	rsunloc = _V((rsunloc.x/veclength), (rsunloc.y/veclength), (rsunloc.z/veclength));//normalized

	VECTOR3 normalab = _V(0, 0, 1);	// no rotation
	VECTOR3 normalcd = _V(0, 0, 1);

	double ab_bga_angle = 2*PI*ABrot_proc; // bga rotation angle
	double cd_bga_angle = 2*PI*CDrot_proc;

	normalab = _V(sin(ab_bga_angle), 0, cos(ab_bga_angle)); // normal to panel after BGA rotation
	normalcd = _V(sin(cd_bga_angle), 0, cos(cd_bga_angle));



	GlobalRot(normalab, normal_ab_ecl);	// change to global reference for future MFD?
	GlobalRot(normalcd, normal_cd_ecl);


	double dot_p_ab_sun = dotp(normalab,rsunloc); // cos of angle beween sun->normal
	double dot_p_cd_sun = dotp(normalcd,rsunloc);

	if (dot_p_ab_sun < 0.0) dot_p_ab_sun = -0.3*dot_p_ab_sun; // more than 90deg from normal
	if (dot_p_cd_sun < 0.0) dot_p_cd_sun = -0.3*dot_p_cd_sun; // back face generates 30% power

	powerab = 29.6832*dot_p_ab_sun;
	powercd = 29.6832*dot_p_cd_sun;


	/////////////// eclipse? ///////////////

	VECTOR3 earth_rel_pos;
	char ebuf[7]= "Earth";
	OBJHANDLE eRef = oapiGetObjectByName(ebuf);
	GetRelativePos(eRef, earth_rel_pos);
	double earth_dist = length(earth_rel_pos);

	double terminator_angle = asin(6376010/earth_dist);

	VECTOR3 sun_rel_pos;
	GetRelativePos(hRef, sun_rel_pos);
	double sun_dist = length(sun_rel_pos);

	double cos_earth_sc_sun_angle = (dotp(sun_rel_pos, earth_rel_pos))/(sun_dist*earth_dist);
	double earth_sc_sun_angle = acos(cos_earth_sc_sun_angle);

	double sep_angle = (earth_sc_sun_angle+0.0045) - terminator_angle;
	double power_adjust = sep_angle/0.009;

	if (power_adjust > 1.0) power_adjust = 1;
	else if (power_adjust < 0.0) power_adjust = 0;

	/////////// earth insolation ///////////////
	double cos_normal_ab_earth = (dotp( -earth_rel_pos, normal_ab_ecl)/earth_dist);
	double cos_normal_cd_earth = (dotp( -earth_rel_pos, normal_cd_ecl)/earth_dist);

	if (cos_normal_ab_earth < 0) cos_normal_ab_earth = -0.3*cos_normal_ab_earth;
	if (cos_normal_cd_earth < 0) cos_normal_cd_earth = -0.3*cos_normal_cd_earth;

	cos_earth_sc_sun_angle=-cos_earth_sc_sun_angle;
	if (cos_earth_sc_sun_angle < 0)cos_earth_sc_sun_angle = 0;

	double ei_power_ab = cos_normal_ab_earth*cos_earth_sc_sun_angle*10.3891;
	double ei_power_cd = cos_normal_cd_earth*cos_earth_sc_sun_angle*10.3891;

	powerab = powerab + ei_power_ab;
	powercd = powercd + ei_power_cd;


	////////////////////////////////////////


	if(solp_status == DOOR_OPEN)powerab = powerab*power_adjust*error_current_ab;
	else powerab = 0.0;
	if(solp2_status == DOOR_OPEN)powercd = powercd*power_adjust*error_current_cd;
	else powercd = 0.0;

	//sprintf (oapiDebugString(), "sep angle %lf", (sep_angle-0.0045)*DEG);

}

void p6::AutoGrapple (void)
{
	{
	             // grapple satellite

		VECTOR3 att_point_pos;
		VECTOR3 att_point_dir;
		VECTOR3 att_point_rot;
		GetAttachmentParams(P6_attachment3, att_point_pos, att_point_dir, att_point_rot);
		VECTOR3 gpos, grms, pos, dir, rot;
		Local2Global (att_point_pos, grms);  // global position of grapple point

		// Search the complete vessel list for a grappling candidate.
		// Not very scalable ...
		for (DWORD i = 0; i < oapiGetVesselCount(); i++) {
			OBJHANDLE hV = oapiGetVesselByIndex (i);
			if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
			oapiGetGlobalPos (hV, &gpos);
			if (dist (gpos, grms) < (oapiGetSize (hV)+ 10)) { // in range
				VESSEL *v = oapiGetVesselInterface (hV);
				DWORD nAttach = v->AttachmentCount (true);
				for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
					ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
					const char *id = v->GetAttachmentId (hAtt);
					if (strncmp (id, "MM", 2)) continue; // attachment point not compatible
					v->GetAttachmentParams (hAtt, pos, dir, rot);
					v->Local2Global (pos, gpos);
					if (dist (gpos, grms) < MAX_GRAPPLING_DIST) { // found one!

/// start release child from previous parent (not automatic? ->Orbiter bug)
						for (DWORD k = 0; k < oapiGetVesselCount(); k++)
							{
							OBJHANDLE hVp = oapiGetVesselByIndex (k);
							if (hVp == GetHandle()) continue;
							if (hVp == hV) continue;
							VESSEL *w = oapiGetVesselInterface (hVp);
							DWORD nAttachp = w->AttachmentCount (false);
							for (DWORD l = 0; l < nAttachp; l++)
								{
								ATTACHMENTHANDLE hAttp = w->GetAttachmentHandle (false, l);
								if (hV == w->GetAttachmentStatus (hAttp))DetachChild (hAttp);
								}
							}
/// end of release child from previous parent
						AttachChild (hV, P6_attachment3, hAtt);

						return;
					}
				}
			}
		}

	}
}

void p6::ReleaseGrapple (void)
{
DetachChild (P6_attachment3);
}
void p6::CheckChildAttach(void)
{
	// check if P5 grappled by Shuttle, if yes- release previous parent
	OBJHANDLE hC = GetAttachmentStatus(P6_attachment3);
	VESSEL *v = oapiGetVesselInterface (hC);
	DWORD nAttach = v->AttachmentCount (true);
	for (DWORD j = 0; j < nAttach; j++)
		{ // now scan all attachment points of the candidate
		ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "GS", 2))continue;
		if(v->GetAttachmentStatus(hAtt))DetachChild (P6_attachment3);
		return;
		}
}


void p6::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC)
{
	// draw the default HUD
	VESSEL2::clbkDrawHUD (mode, hps, hDC);

	double s = ((hps->H));
	double d = (s/768);


	int wc = (int)(148*d);
	int w = (int)(164*d);
	int w1 = (int) (180*d);
	int w2 = (int)(196*d);
	int w3 = (int)(212*d);
	int w4 = (int)(228*d);
	int w5 = (int)(244*d);
	int w6 = (int)(260*d);
	int w7 = (int)(276*d);
	int w8 = (int)(292*d);
	int w9 = (int)(308*d);
	int w10 = (int)(324*d);
	int w11 = (int)(340*d);

	///////// control header //////////

	{
	SetTextColor(hDC,WHITE);
	char abuf[256];
	sprintf(abuf,"Controls:");
	TextOut(hDC,10,wc,abuf,strlen(abuf));
	}


	////////////// panels //////////////////////

	if (deploy_status == 0)
	{
	SetTextColor(hDC,GREEN);
	char abuf[256];
	sprintf(abuf,"G = Deploy Solar Panels");
	TextOut(hDC,10,w,abuf,strlen(abuf));
	}
	else if (deploy_status == 1)
	{
	SetTextColor(hDC,GREEN);
	char bbuf[256];
	sprintf(bbuf,"G = Pause Solar Panels Deploy");
	TextOut(hDC,10,w,bbuf,strlen(bbuf));
	}
	else if (deploy_status == 2)
	{
	SetTextColor(hDC,GREEN);
	char bbuf[256];
	sprintf(bbuf,"G = Stow Solar Panels");
	TextOut(hDC,10,w,bbuf,strlen(bbuf));
	}
	else if (deploy_status == 3)
	{
	SetTextColor(hDC,GREEN);
	char bbuf[256];
	sprintf(bbuf,"G = Pause Solar Panels Stow");
	TextOut(hDC,10,w,bbuf,strlen(bbuf));
	}
	else if (deploy_status == 4)
	{
	SetTextColor(hDC,GREEN);
	char bbuf[256];
	sprintf(bbuf,"G = Continue Solar Panels Deploy");
	TextOut(hDC,10,w,bbuf,strlen(bbuf));
	}
	else if (deploy_status == 5)
	{
	SetTextColor(hDC,GREEN);
	char bbuf[256];
	sprintf(bbuf,"G = Continue Solar Panels Stow");
	TextOut(hDC,10,w,bbuf,strlen(bbuf));
	}

	/////////////////// auto sun tracking //////////////

	if (auto_sw == 0 && deploy_status == 2)
	{
	SetTextColor(hDC,GREEN);
	char abuf[256];
	sprintf(abuf,"K = Sun Tracking On/Off");
	TextOut(hDC,10,w1,abuf,strlen(abuf));
	}
	else if (auto_sw == 0)
	{
	SetTextColor(hDC,RED);
	char abuf[256];
	sprintf(abuf,"K = Sun Tracking On/Off");
	TextOut(hDC,10,w1,abuf,strlen(abuf));
	}
	else
	{
	SetTextColor(hDC,YELLOW);
	char abuf[256];
	sprintf(abuf,"K = Sun Tracking On/Off");
	TextOut(hDC,10,w1,abuf,strlen(abuf));
	}


	//////////// panel AB
	if (auto_sw == 1)
	{
	SetTextColor(hDC,RED);
	char abuf[256];
	sprintf(abuf,"NumPad 4,5,6 = Rotate AB BGA Left/Stop/Right");
	TextOut(hDC,10,w3,abuf,strlen(abuf));
	}
	else if (deploy_status < 2)
	{
	SetTextColor(hDC,RED);
	char abuf[256];
	sprintf(abuf,"NumPad 4,5,6 = Rotate AB BGA Left/Stop/Right");
	TextOut(hDC,10,w3,abuf,strlen(abuf));
	}
	else if (deploy_status > 2)
	{
	SetTextColor(hDC,RED);
	char abuf[256];
	sprintf(abuf,"NumPad 4,5,6 = Rotate AB BGA Left/Stop/Right");
	TextOut(hDC,10,w3,abuf,strlen(abuf));
	}
	else if (rot_sw == 0)
	{
	SetTextColor(hDC,GREEN);
	char abuf[256];
	sprintf(abuf,"NumPad 4,5,6 = Rotate AB BGA Left/Stop/Right");
	TextOut(hDC,10,w3,abuf,strlen(abuf));
	}
	else
	{
	SetTextColor(hDC,YELLOW);
	char abuf[256];
	sprintf(abuf,"NumPad 4,5,6 = Rotate AB BGA Left/Stop/Right");
	TextOut(hDC,10,w3,abuf,strlen(abuf));
	}

	///////////////// panels CD
	if (auto_sw == 1)
	{
	SetTextColor(hDC,RED);
	char abuf[256];
	sprintf(abuf,"NumPad 7,8,9 = Rotate CD BGA Left/Stop/Right");
	TextOut(hDC,10,w4,abuf,strlen(abuf));
	}
	else if (deploy_status < 2)
	{
	SetTextColor(hDC,RED);
	char abuf[256];
	sprintf(abuf,"NumPad 7,8,9 = Rotate CD BGA Left/Stop/Right");
	TextOut(hDC,10,w4,abuf,strlen(abuf));
	}
	else if (deploy_status > 2)
	{
	SetTextColor(hDC,RED);
	char abuf[256];
	sprintf(abuf,"NumPad 7,8,9 = Rotate CD BGA Left/Stop/Right");
	TextOut(hDC,10,w4,abuf,strlen(abuf));
	}
	else if (rot_sw2 == 0)
	{
	SetTextColor(hDC,GREEN);
	char abuf[256];
	sprintf(abuf,"NumPad 7,8,9 = Rotate CD BGA Left/Stop/Right");
	TextOut(hDC,10,w4,abuf,strlen(abuf));
	}
	else
	{
	SetTextColor(hDC,YELLOW);
	char abuf[256];
	sprintf(abuf,"NumPad 7,8,9 = Rotate CD BGA Left/Stop/Right");
	TextOut(hDC,10,w4,abuf,strlen(abuf));
	}

	/////////////////// attachment

	if (GetAttachmentStatus (P6_attachment3))
	{
	SetTextColor(hDC,GREEN);
	char abuf[256];
	sprintf(abuf,"M = Release Parent Attachment Point");
	TextOut(hDC,10,w5,abuf,strlen(abuf));
	}
	else
	{
	SetTextColor(hDC,GREEN);
	char abuf[256];
	sprintf(abuf,"N = Grapple Parent Attachment Point");
	TextOut(hDC,10,w5,abuf,strlen(abuf));
	}

	///////////////// output

	{
	SetTextColor(hDC,WHITE);
	char abuf[256];
	sprintf(abuf,"Power Output:");
	TextOut(hDC,10,w8,abuf,strlen(abuf));
	}

	{
	SetTextColor(hDC,GREEN);
	char abuf[256];
	sprintf(abuf,"AB Panels:  %0.3f kW", powerab);
	TextOut(hDC,10,w9,abuf,strlen(abuf));
	}
	{
	SetTextColor(hDC,GREEN);
	char abuf[256];
	sprintf(abuf,"CD Panels:  %0.3f kW", powercd);
	TextOut(hDC,10,w10,abuf,strlen(abuf));
	}


}
void p6::clbkPostStep (double simt, double simdt, double mjd)
{

	if (deploy_status < 4)
	{
	// Animate motors
	if (motors_anim_status >= DOOR_CLOSING) {
		double da = simdt * BGA_DEPLOY_SPEED;
		if (motors_anim_status == DOOR_CLOSING) {
			if (motors_proc > 0.0) motors_proc = max (0.0, motors_proc-da);
			else
			{
			motors_anim_status = DOOR_CLOSED;
			deploy_status = 0;
			}
		} else {
			if (motors_proc < 1.0) motors_proc = min (1.0, motors_proc+da);
			else                motors_anim_status = DOOR_OPEN;
		}
		SetAnimation (motors_anim, motors_proc);
	}



	// Animate ABsolpanels
	if (solp_status >= DOOR_CLOSING) {
		double da = simdt * SOLP_DEPLOY_SPEED;
		if (solp_status == DOOR_CLOSING) {
			if (solp_proc > 0.0) solp_proc = max (0.0, solp_proc-da);
			else                solp_status = DOOR_CLOSED;
		} else {
			if (solp_proc < 1.0) solp_proc = min (1.0, solp_proc+da);
			else                solp_status = DOOR_OPEN;
		}
		SetAnimation (anim_solp, solp_proc);
	}

	// Animate CDsolpanels
	if (solp2_status >= DOOR_CLOSING) {
		double da = simdt * SOLP_DEPLOY_SPEED;
		if (solp2_status == DOOR_CLOSING) {
			if (solp2_proc > 0.0) solp2_proc = max (0.0, solp2_proc-da);
			else                solp2_status = DOOR_CLOSED;
		} else {
			if (solp2_proc < 1.0) solp2_proc = min (1.0, solp2_proc+da);
			else                solp2_status = DOOR_OPEN;
		}
		SetAnimation (anim_solp2, solp2_proc);
	}


	// animate AB truss
	if (solp_status > 1)
	{
	AB_truss_proc = sin((0.5*PI)* solp_proc);
	SetAnimation (AB_truss_anim, AB_truss_proc);
	}

	// animate CD truss
	if (solp2_status > 1)
	{
	CD_truss_proc = sin((0.5*PI)* solp2_proc);
	SetAnimation (CD_truss_anim, CD_truss_proc);
	}


	// Animate Rad
	if (rad_status >= DOOR_CLOSING) {
		double da = simdt * RAD_DEPLOY_SPEED;
		if (rad_status == DOOR_CLOSING) {
			if (rad_proc > 0.0) rad_proc = max (0.0, rad_proc-da);
			else                rad_status = DOOR_CLOSED;
		} else {
			if (rad_proc < 1.0) rad_proc = min (1.0, rad_proc+da);
			else
			{
			rad_status = DOOR_OPEN;
			deploy_status = 2;
			}
		}
		SetAnimation (rad_anim, rad_proc);
	}

	}

	// animate AB panels rotate
	if (rot_sw == 1)
	{
	double da = simdt * SOLP_ROT_SPEED;
	ABrot_proc = ABrot_proc + da;
	if (ABrot_proc > 1) ABrot_proc = ABrot_proc - 1;
	if (ABrot_proc < 0) ABrot_proc = ABrot_proc + 1;
	SetAnimation (ABrot_anim, ABrot_proc);
	}

	if (rot_sw == 2)
	{
	double da = simdt * SOLP_ROT_SPEED;
	ABrot_proc = ABrot_proc - da;
	if (ABrot_proc > 1) ABrot_proc = ABrot_proc - 1;
	if (ABrot_proc < 0) ABrot_proc = ABrot_proc + 1;
	SetAnimation (ABrot_anim, ABrot_proc);
	}

	// animate CD panels rotate
	if (rot_sw2 == 1)
	{
	double da = simdt * SOLP_ROT_SPEED;
	CDrot_proc = CDrot_proc + da;
	if (CDrot_proc > 1) CDrot_proc = CDrot_proc - 1;
	if (CDrot_proc < 0) CDrot_proc = CDrot_proc + 1;
	SetAnimation (CDrot_anim, CDrot_proc);
	}

	if (rot_sw2 == 2)
	{
	double da = simdt * SOLP_ROT_SPEED;
	CDrot_proc = CDrot_proc - da;
	if (CDrot_proc > 1) CDrot_proc = CDrot_proc - 1;
	if (CDrot_proc < 0) CDrot_proc = CDrot_proc + 1;
	SetAnimation (CDrot_anim, CDrot_proc);
	}



	/////////////// reset panels /////////////////
	if (deploy_status == 3)
	{
	// animate AB panels reset
	if (ABrot_proc > 0.5)
	{
	double da = simdt * SOLP_ROT_SPEED;
	ABrot_proc = ABrot_proc + da;
	if (ABrot_proc > 1) ABrot_proc = 0.0;
	SetAnimation (ABrot_anim, ABrot_proc);
	}

	if (ABrot_proc < 0.5 && ABrot_proc > 0.0)
	{
	double da = simdt * SOLP_ROT_SPEED;
	ABrot_proc = ABrot_proc - da;
	if (ABrot_proc < 0) ABrot_proc = 0.0;
	SetAnimation (ABrot_anim, ABrot_proc);
	}

	// animate CD panels rest
	if (CDrot_proc > 0.5)
	{
	double da = simdt * SOLP_ROT_SPEED;
	CDrot_proc = CDrot_proc + da;
	if (CDrot_proc > 1) CDrot_proc = 0.0;
	SetAnimation (CDrot_anim, CDrot_proc);
	}

	if (CDrot_proc < 0.5 && CDrot_proc > 0.0)
	{
	double da = simdt * SOLP_ROT_SPEED;
	CDrot_proc = CDrot_proc - da;
	if (CDrot_proc < 0) CDrot_proc = 0.0;
	SetAnimation (CDrot_anim, CDrot_proc);
	}
	}



	if (solp2_status == DOOR_OPEN)CalculatePower();
	else if (solp_status == DOOR_OPEN)CalculatePower();
	else
	{
	GlobalRot(_V(0,0,1),normal_ab_ecl);
	GlobalRot(_V(0,0,1),normal_cd_ecl);
	powerab = 0.0;
	powercd = 0.0;
	}


	///////// 3 deg pointing error
	//// error AB

	if (err_sw_ab == 0)
	{
	error_target_ab = (1 - 0.002*(oapiRand ()));
	err_sw_ab = 1;
	}

	if (error_current_ab >= error_target_ab)
		{
		error_current_ab = error_current_ab-0.0000002;
		if (error_current_ab < error_target_ab) err_sw_ab = 0;
		}
	else if (error_current_ab < error_target_ab)
		{
		error_current_ab = error_current_ab+0.0000002;
		if (error_current_ab > error_target_ab) err_sw_ab = 0;
		}

	//// error CD
	if (err_sw_cd == 0)
	{
	error_target_cd = (1 - 0.002*(oapiRand ()));
	err_sw_cd = 1;
	}

	if (error_current_cd >= error_target_cd)
		{
		error_current_cd = error_current_cd-0.0000002;
		if (error_current_cd < error_target_cd) err_sw_cd = 0;
		}
	else if (error_current_cd < error_target_cd)
		{
		error_current_cd = error_current_cd+0.0000002;
		if (error_current_cd > error_target_cd) err_sw_cd = 0;
		}

	///// tot up total power and round off
	power_tot = (int)(1000*(powerab+powercd));
	if (((1000*(powerab+powercd)) - power_tot) >= 0.5)power_tot = power_tot+1;// round off

}